import * as dotenv from 'dotenv';
import {Guid} from 'guid-typescript';
import {Event} from './event.interface';
import {NachklangCalendarDB} from '../Calendar.db';


dotenv.config();

/**
 * Returns all events for the given calendar
 * @param calendarId The calendar Id
 */
export const getAllEvents = async (calendarId: number): Promise<Event[]> => {
	let conn = await NachklangCalendarDB.getConnection();
	let eventRows: Event[] = [];
	try {
		const eventsQuery = `
		SELECT e.calendar_id, u.full_name, v.* FROM events e
			INNER JOIN (
  				SELECT event_id, MAX(event_version_id) AS latest_version
			  	FROM event_versions
			  	GROUP BY event_id
			) latest_versions
			ON e.event_id = latest_versions.event_id
			INNER JOIN event_versions v
			ON v.event_id = latest_versions.event_id AND v.event_version_id = latest_versions.latest_version
			LEFT OUTER JOIN users u ON u.user_id = e.created_by_id
			WHERE e.calendar_id = ? AND v.status = 'PUBLIC'
			ORDER BY e.event_id`;
		const eventsRes = await conn.query(eventsQuery, calendarId);

		for (let row of eventsRes) {
			eventRows.push({
				eventId: row.event_id,
				calendarId: row.calendar_id,
				uuid: row.uuid,
				name: row.name,
				description: row.description,
				startDateTime: row.start_datetime,
				endDateTime: row.end_datetime,
				createdDate: row.created_date,
				location: row.location,
				createdBy: row.full_name,
				createdById: row.created_by_id,
				url: row.url,
				wholeDay: row.whole_day
			});
		}

		return eventRows;
	} catch (err) {
		throw err;
	} finally {
		// Return connection
		await conn.end();
	}
};

export const getAllEventsAdmin = async (calendarId: number): Promise<Event[]> => {
	let conn = await NachklangCalendarDB.getConnection();
	let eventRows: Event[] = [];
	try {
		const eventsQuery = `
		SELECT e.calendar_id, u.full_name, v.* FROM events e
			INNER JOIN (
  				SELECT event_id, MAX(event_version_id) AS latest_version
			  	FROM event_versions
			  	GROUP BY event_id
			) latest_versions
			ON e.event_id = latest_versions.event_id
			INNER JOIN event_versions v
			ON v.event_id = latest_versions.event_id AND v.event_version_id = latest_versions.latest_version
			LEFT OUTER JOIN users u ON u.user_id = e.created_by_id
			WHERE e.calendar_id = ?
			ORDER BY e.event_id`;
		const eventsRes = await conn.query(eventsQuery, calendarId);

		for (let row of eventsRes) {
			eventRows.push({
				eventId: row.event_id,
				calendarId: row.calendar_id,
				uuid: row.uuid,
				name: row.name,
				description: row.description,
				startDateTime: row.start_datetime,
				endDateTime: row.end_datetime,
				createdDate: row.created_date,
				location: row.location,
				createdBy: row.full_name,
				createdById: row.created_by_id,
				url: row.url,
				wholeDay: row.whole_day,
				status: row.status
			});
		}

		return eventRows;
	} catch (err) {
		throw err;
	} finally {
		// Return connection
		await conn.end();
	}
};

/**
 * Create the given event in the database
 * @param event The event to create
 */
export const createEvent = async (event: Event): Promise<number> => {
	let conn = await NachklangCalendarDB.getConnection();
	try {
		let eventUUID = Guid.create().toString();
		const eventsQuery = 'INSERT INTO events (calendar_id, uuid, created_by_id) VALUES (?,?,?) RETURNING event_id';
		const eventsRes = await conn.execute(eventsQuery, [event.calendarId, eventUUID, event.createdById]);

		const versionQuery = 'INSERT INTO event_versions (event_id, name, description, start_datetime, end_datetime, whole_day, location, url, status, version_created_by_id) VALUES (?,?,?,?,?,?,?,?,?,?);'
		const versionRes = await conn.execute(versionQuery, [eventsRes[0].event_id, event.name, event.description, event.startDateTime, event.endDateTime, event.wholeDay, event.location, event.url, event.status, event.createdById]);

		await conn.commit();

		return eventsRes[0].event_id;
	} catch (err) {
		await conn.rollback();
		throw err;
	} finally {
		// Return connection
		await conn.commit();
		await conn.end();
	}
};

/**
 * Update the given event in the database
 * @param event The event to update
 */
export const updateEvent = async (event: Event): Promise<number> => {
	let conn = await NachklangCalendarDB.getConnection();
	try {
		const versionQuery = 'INSERT INTO event_versions (event_id, name, description, start_datetime, end_datetime, whole_day, location, url, status, version_created_by_id) VALUES (?,?,?,?,?,?,?,?,?,?);'
		const versionRes = await conn.execute(versionQuery, [event.eventId, event.name, event.description, event.startDateTime, event.endDateTime, event.wholeDay, event.location, event.url, event.status, event.createdById]);

		await conn.commit();

		return versionRes.affectedRows;
	} catch (err) {
		await conn.rollback();
		throw err;
	} finally {
		// Return connection
		await conn.commit();
		await conn.end();
	}
};

/**
 * Deletes the given event from the database
 * @param event The event to delete
 */
export const deleteEvent = async (event: Event): Promise<boolean> => {
	let conn = await NachklangCalendarDB.getConnection();
	try {
		const versionQuery = 'INSERT INTO event_versions (event_id, status, version_created_by_id) VALUES (?,?,?);'
		const versionRes = await conn.execute(versionQuery, [event.eventId, 'DELETED', event.createdById]);

		await conn.commit();

		return versionRes.affectedRows === 1;
	} catch (err) {
		await conn.rollback();
		throw err;
	} finally {
		// Return connection
		await conn.commit();
		await conn.end();
	}
};
